/*
 * ** Network and Service Differentiation Extensions to GridSim 3.0 **
 *
 * Gokul Poduval & Chen-Khong Tham
 * Computer Communication Networks (CCN) Lab
 * Dept of Electrical & Computer Engineering
 * National University of Singapore
 * August 2004
 *
 * Licence: GPL - http://www.gnu.org/copyleft/gpl.html
 *
 * FIFOScheduler.java - Implements a First in First Out scheduler
 *
 */

package gridsim.net;

import eduni.simjava.*;
import gridsim.*;
import java.util.*;


/**
 * FIFOScheduler implements a First in First Out Scheduler. This means that
 * all the packets are enqued at the tail of a queue, and packets depart from
 * the head of the queue. Packets are not reordered, and no differentiated
 * service will be provided.
 *
 * @author Gokul Poduval & Chen-Khong Tham, National University of Singapore
 * @since  GridSim Toolkit 3.1
 * @invariant $none
 */
public class FIFOScheduler extends PacketScheduler
{
    private Vector pktList; /* Sorted List of all Packets */


    /**
     * Creates a new FIFO packet scheduler with the specified name and baud rate
     * (in bits/s). The name can be useful for debugging purposes, but serves
     * no functional purpose.
     * @param name       Name of this scheduler
     * @param baudRate   baud rate in bits/s of the port that is using
     *                   this scheduler.
     * @throws ParameterException This happens when the name is null or
     *                   the baud rate <= 0
     * @pre name != null
     * @pre baudRate > 0
     * @post $none
     */
    public FIFOScheduler(String name, double baudRate) 
                         throws ParameterException
    {
        super(name, baudRate);
        pktList = new Vector();
    }

    /**
     * Creates a new FIFO packet scheduler with the specified baud rate
     * (bits/s). The name is set to a generic <b>"FIFOScheduler"</b>.
     *
     * @param baudRate   baud rate in bits/s of the port that is using
     *                   this scheduler.
     * @throws ParameterException This happens when the baud rate <= 0
     * @pre baudRate > 0
     * @post $none
     */
    public FIFOScheduler(double baudRate) throws ParameterException
    {
        super("FIFOScheduler", baudRate);
        pktList = new Vector();
    }

    /**
     * Creates a new FIFO packet scheduler with the specified name.
     * The baud rate is
     * left at 0, and should be set with
     * {@link gridsim.net.PacketScheduler#setBaudRate(double)}
     * before being the simulation is started.
     *
     * @param name Name of this scheduler
     * @throws ParameterException This happens when the name is null
     * @see gridsim.net.PacketScheduler#setBaudRate(double)
     * @pre name != null
     * @post $none
     */
    public FIFOScheduler(String name) throws ParameterException
    {
        super(name);
        pktList = new Vector();
    }

    /**
     * Creates a new packet scheduler with the name <b>"FIFOScheduler"</b>.
     * The baud rate is left at 0, and should be set with
     * {@link gridsim.net.PacketScheduler#setBaudRate(double)}
     * before being the simulation is started.
     * @throws ParameterException This happens when the name is null
     * @see gridsim.net.PacketScheduler#setBaudRate(double)
     * @pre $none
     * @post $none
     */
    public FIFOScheduler() throws ParameterException
    {
        super("FIFOScheduler");
        pktList = new Vector();
    }

    /**
     * In this scheduler, the packet is put into the tail of the queue.
     * There is no buffer management, so packets are never dropped, and the
     * queue can grow as long as system memory is available.
     *
     * @param np    A Packet to be enqued by this scheduler.
     * @return <tt>true</tt> if enqued, <tt>false</tt> otherwise
     * @pre np != null
     * @post $none
     */
    public synchronized boolean enque(Packet np)
    {
        pktList.add(np);
        return true;
    }

    /**
     * In this scheduler, the packet returned is always from the head of the
     * queue.
     *
     * @return the packet to be sent out, or <tt>null</tt> if a list is empty
     * @pre $none
     * @post $none
     */
    public synchronized Packet deque()
    {
        if (pktList.size() == 0) {
            return null;
        }
        
        Packet p = (Packet) pktList.remove(0);
        return p;
    }

    /**
     * Determines whether the scheduler is currently keeping any packets in
     * its queue(s).
     *
     * @return <tt>true</tt> if no packets are enqueued, <tt>false</tt>
     *         otherwise
     * @pre $none
     * @post $none
     */
    public synchronized boolean isEmpty() {
        return pktList.isEmpty();
    }

    /**
     * Determines the number of packets that are currently enqueued in this
     * scheduler.
     *
     * @return the number of packets enqueud by this scheduler.
     * @pre $none
     * @post $none
     */
    public synchronized int size() {
        return pktList.size();
    }

} // end class

